# Hands-On Ensemble Learning with Python

<a href="https://www.packtpub.com/data/hands-on-ensemble-learning-with-python"><img src="https://www.packtpub.com/media/catalog/product/cache/e4d64343b1bc593f1c5348fe05efa4a6/9/7/9781789612851-original.png" alt="Hands-On Ensemble Learning with Python" height="256px" align="right"></a>

This is the code repository for [Hands-On Ensemble Learning with Python](https://www.packtpub.com/data/hands-on-ensemble-learning-with-python), published by Packt.

**Build highly optimized ensemble machine learning models using scikit-learn and Keras**

## What is this book about?
Ensembling is a technique for combining two or more similar or dissimilar machine learning algorithms to create a model that delivers superior predictive power. This book will demonstrate how you can use a variety of weak algorithms to make a strong predictive model.

With its hands-on approach, you'll not only get up to speed on the basic theory, but also the application of various ensemble learning techniques. Using examples and real-world datasets, you'll be able to produce better machine learning models to solve supervised learning problems such as classification and regression. Later in the book, you'll go on to leverage ensemble learning techniques such as clustering to produce unsupervised machine learning models. As you progress, the chapters will cover different machine learning algorithms that are widely used in the practical world to make predictions and classifications. You'll even get to grips with using Python libraries such as scikit-learn and Keras to implement different ensemble models.

By the end of this book, you will be well versed in ensemble learning and have the skills you need to understand which ensemble method is required for which problem, in order to successfully implement them in real-world scenarios.

This book covers the following exciting features:
* Implement ensemble methods to generate models with high accuracy
* Overcome challenges such as bias and variance
* Explore machine learning algorithms to evaluate model performance
* Understand how to construct, evaluate, and apply ensemble models
* Analyze tweets in real time using Twitter's streaming API
* Use Keras to build an ensemble of neural networks for the MovieLens dataset

If you feel this book is for you, get your [copy](https://www.amazon.com/dp/1789612853) today!

<a href="https://www.packtpub.com/?utm_source=github&utm_medium=banner&utm_campaign=GitHubBanner"><img src="https://raw.githubusercontent.com/PacktPublishing/GitHub/master/GitHub.png" 
alt="https://www.packtpub.com/" border="5" /></a>

## Instructions and Navigations
All of the code is organized into folders. For example, Chapter01.

The code will look like the following:
```
# Accuracy of hard voting
print('-'*30)
print('Hard Voting:', accuracy_score(y_test, hard_predictions))
```

**Following is what you need for this book:**
This book is for data analysts, data scientists, machine learning engineers, and other professionals who are looking to generate advanced models using ensemble techniques. Some understanding of machine learning concepts, Python programming and AWS will be beneficial.	

With the following software and hardware list you can run all code files present in the book (Chapter 1-11).
### Software and Hardware List
| Chapter | Software required | OS required |
| -------- | ------------------------------------ | ----------------------------------- |
| All | Python(Jupyter notebook) | Windows, Mac OS X, and Linux (Any) |

We also provide a PDF file that has color images of the screenshots/diagrams used in this book. [Click here to download it](https://static.packt-cdn.com/downloads/9781789612851_ColorImages.pdf).

### Related products
* Ensemble Machine Learning Cookbook [[Packt]](https://www.packtpub.com/big-data-and-business-intelligence/ensemble-machine-learning-cookbook) [[Amazon]](https://www.amazon.com/dp/1789136601)


## Get to Know the Author
**George Kyriakides** is a Ph.D. researcher, studying distributed neural architecture search. His interests and experience include the automated generation and optimization of predictive models for a wide array of applications, such as image recognition, time series analysis, and financial applications. He holds an M.Sc. in computational methods and applications, and a B.Sc. in applied informatics, both from the University of Macedonia, Thessaloniki, Greece.

**Konstantinos G. Margaritis** has been a teacher and researcher in computer science for more than 30 years. His research interests include parallel and distributed computing, as well as computational intelligence and machine learning. He holds an M.Eng. in electrical engineering (Aristotle University of Thessaloniki, Greece), as well as an M.Sc. and a Ph.D. in computer science (Loughborough University, UK). He is a professor at the Department of Applied Informatics, University of Macedonia, Thessaloniki, Greece. 


### Suggestions and Feedback
[Click here](https://docs.google.com/forms/d/e/1FAIpQLSdy7dATC6QmEL81FIUuymZ0Wy9vH1jHkvpY57OiMeKGqib_Ow/viewform) if you have any feedback or suggestions.


